"""
Dictionary of CSS3 extended color keyword names.

Source: https://www.w3.org/wiki/CSS3/Color/Extended_color_keywords
"""

COLORS = {
    "aliceblue": 0xF0F8FF,
    "antiquewhite": 0xFAEBD7,
    "aqua": 0x00FFFF,
    "aquamarine": 0x7FFFD4,
    "azure": 0xF0FFFF,
    "beige": 0xF5F5DC,
    "bisque": 0xFFE4C4,
    "black": 0x000000,
    "blanchedalmond": 0xFFEBCD,
    "blue": 0x0000FF,
    "blueviolet": 0x8A2BE2,
    "brown": 0xA52A2A,
    "burlywood": 0xDEB887,
    "cadetblue": 0x5F9EA0,
    "chartreuse": 0x7FFF00,
    "chocolate": 0xD2691E,
    "coral": 0xFF7F50,
    "cornflowerblue": 0x6495ED,
    "cornsilk": 0xFFF8DC,
    "crimson": 0xDC143C,
    "cyan": 0x00FFFF,
    "darkblue": 0x00008B,
    "darkcyan": 0x008B8B,
    "darkgoldenrod": 0xB8860B,
    "darkgray": 0xA9A9A9,
    "darkgreen": 0x006400,
    "darkgrey": 0xA9A9A9,
    "darkkhaki": 0xBDB76B,
    "darkmagenta": 0x8B008B,
    "darkolivegreen": 0x556B2F,
    "darkorange": 0xFF8C00,
    "darkorchid": 0x9932CC,
    "darkred": 0x8B0000,
    "darksalmon": 0xE9967A,
    "darkseagreen": 0x8FBC8F,
    "darkslateblue": 0x483D8B,
    "darkslategray": 0x2F4F4F,
    "darkslategrey": 0x2F4F4F,
    "darkturquoise": 0x00CED1,
    "darkviolet": 0x9400D3,
    "deeppink": 0xFF1493,
    "deepskyblue": 0x00BFFF,
    "dimgray": 0x696969,
    "dimgrey ": 0x696969,
    "dodgerblue": 0x1E90FF,
    "firebrick": 0xB22222,
    "floralwhite": 0xFFFAF0,
    "forestgreen": 0x228B22,
    "fuchsia": 0xFF00FF,
    "gainsboro": 0xDCDCDC,
    "ghostwhite": 0xF8F8FF,
    "gold": 0xFFD700,
    "goldenrod": 0xDAA520,
    "gray": 0x808080,
    "green": 0x008000,
    "greenyellow": 0xADFF2F,
    "grey": 0x808080,
    "honeydew": 0xF0FFF0,
    "hotpink": 0xFF69B4,
    "indianred": 0xCD5C5C,
    "indigo": 0x4B0082,
    "ivory": 0xFFFFF0,
    "khaki": 0xF0E68C,
    "lavender": 0xE6E6FA,
    "lavenderblush": 0xFFF0F5,
    "lawngreen": 0x7CFC00,
    "lemonchiffon": 0xFFFACD,
    "lightblue": 0xADD8E6,
    "lightcoral": 0xF08080,
    "lightcyan": 0xE0FFFF,
    "lightgoldenrodyellow": 0xFAFAD2,
    "lightgray": 0xD3D3D3,
    "lightgreen": 0x90EE90,
    "lightgrey": 0xD3D3D3,
    "lightpink": 0xFFB6C1,
    "lightsalmon": 0xFFA07A,
    "lightseagreen": 0x20B2AA,
    "lightskyblue": 0x87CEFA,
    "lightslategray": 0x778899,
    "lightslategrey": 0x778899,
    "lightsteelblue": 0xB0C4DE,
    "lightyellow": 0xFFFFE0,
    "lime": 0x00FF00,
    "limegreen": 0x32CD32,
    "linen": 0xFAF0E6,
    "magenta": 0xFF00FF,
    "maroon": 0x800000,
    "mediumaquamarine": 0x66CDAA,
    "mediumblue": 0x0000CD,
    "mediumorchid": 0xBA55D3,
    "mediumpurple": 0x9370DB,
    "mediumseagreen": 0x3CB371,
    "mediumslateblue": 0x7B68EE,
    "mediumspringgreen": 0x00FA9A,
    "mediumturquoise": 0x48D1CC,
    "mediumvioletred": 0xC71585,
    "midnightblue": 0x191970,
    "mintcream": 0xF5FFFA,
    "mistyrose": 0xFFE4E1,
    "moccasin": 0xFFE4B5,
    "navajowhite": 0xFFDEAD,
    "navy": 0x000080,
    "oldlace": 0xFDF5E6,
    "olive": 0x808000,
    "olivedrab": 0x6B8E23,
    "orange": 0xFFA500,
    "orangered": 0xFF4500,
    "orchid": 0xDA70D6,
    "palegoldenrod": 0xEEE8AA,
    "palegreen": 0x98FD98,
    "paleturquoise": 0xAFEEEE,
    "palevioletred": 0xDB7093,
    "papayawhip": 0xFFEFD5,
    "peachpuff": 0xFFDAB9,
    "peru": 0xCD853F,
    "pink": 0xFFC0CD,
    "plum": 0xDDA0DD,
    "powderblue": 0xB0E0E6,
    "purple": 0x800080,
    "red": 0xFF0000,
    "rosybrown": 0xBC8F8F,
    "royalblue": 0x4169E1,
    "saddlebrown": 0x8B4513,
    "salmon": 0xFA8072,
    "sandybrown": 0xF4A460,
    "seagreen": 0x2E8B57,
    "seashell": 0xFFF5EE,
    "sienna": 0xA0522D,
    "silver": 0xC0C0C0,
    "skyblue": 0x87CEEB,
    "slateblue": 0x6A5ACD,
    "slategray": 0x708090,
    "slategrey": 0x708090,
    "snow": 0xFFFAFA,
    "springgreen": 0x00FF7F,
    "steelblue": 0x4682B4,
    "tan": 0xD2B48C,
    "teal": 0x008080,
    "thistle": 0xD8BFD8,
    "tomato": 0xFF6347,
    "turquoise": 0x40E0D0,
    "violet": 0xEE82EE,
    "wheat": 0xF5DEB3,
    "white": 0xFFFFFF,
    "whitesmoke": 0xF5F5F5,
    "yellow": 0xFFFF00,
    "yellowgreen": 0x9ACD32,
}



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['verna']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'verna',
    'version': '0.0.1',
    'description': 'A module to handle colors',
    'long_description': '# Verna\n\n<a href="https://pypi.org/project/verna"><img alt="PyPI" src="https://img.shields.io/pypi/v/verna"></a>\n<img alt="Build Status" src="https://api.travis-ci.com/ju-sh/verna.svg?branch=master"></img>\n<a href="https://github.com/ju-sh/verna/blob/master/LICENSE.md"><img alt="License: MIT" src="https://img.shields.io/pypi/l/verna"></a>\n\nA simple module to handle colors\n\nOnly RGBA colors are supported at the moment.\n\n---\n\n<h2>Installation</h2>\n\nYou need Python>=3.6 to use Verna.\n\nIt can be installed from PyPI with pip using\n\n    pip install verna\n\n---\n\n<h2>Usage</h2>\n\nColors are represented using objects of class `Color`.\n\nThe color value is essentially stored as an integer as the `integer` attribute.\n\nThe following properties can be used to access the different color components.\n\n    color.alpha\n    color.red\n    color.green\n    color.blue\n\nwhere `color` is an instance of `Color`.\n\nThe different color components can be edited with one of the following values\n\n - a percentage in string form with a \'%\' at the end (eg: "20%")\n - an integer from 0 to 255 (eg: 0xff, 255)\n - a float from 0.0 to 1.0 (only for `alpha` property. eg: 0.4)\n\nSo, the following are valid:\n\n    color = Color(0x00ffff)\n    color.alpha = 0x80\n    color.alpha = "50%"\n    color.alpha = 0.5   # float values can be assigned\n                        # only to alpha property\n\n    color.red = 0xff    # Same as color.red = 255\n    color.red = "100%"\n    color.green = 217\n    color.green = "85%"\n    color.blue = 0xf5\n    color.blue = "96%"\n\nwhereas the following will cause error:\n\n    color = Color(0x00ffff)\n    color.alpha = 0x1ff    # > 0xff\n    color.alpha = -1       # < 0.0\n    color.alpha = "120%"   # > 100%\n    color.alpha = "120"    # No \'%\' at end\n    color.alpha = 1.2      # > 1.0\n    color.alpha = True     # Invalid type: bool\n    color.red = 0.5        # float value accepted only for alpha\n\nA `Color` object may be created in multiple ways.\n\nBy default, alpha value is `0`.\n\n<h6>From integer color code</h6>\n\nFor example, cyan (solid), which has color code `0x00ffff` can be created like\n\n    Color(0x00ffff)\n\nwhich is same as\n\n    Color(65535)\n\n<h6>From color name</h6>\n\n`Color.from_name()` can be used to create `Color` objects from a [CSS3 color name](https://www.w3.org/wiki/CSS3/Color/Extended_color_keywords).\n\nFor example, cyan can be created with\n\n    Color.from_name(\'cyan\')\n\n<h6>From RGBA values</h6>\n\n`Color.from_rgba()` can be used to create an instance of `Color` from RGBA values.\n\n    Color.from_rgba(255, 255, 0)         # solid yellow\n    Color.from_rgba(255, 255, 0, 0.5)    # translucent yellow\n\n',
    'author': 'Julin S',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ju-sh/verna',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""
Data structures used to handle colors in Verna
"""

from typing import Union, List

from verna import names


class Color(int):
    """
    Represents a color.

    Only RGB scheme is supported.
    """
    def __init__(self, integer: int):
        super().__init__()
        if integer < 0 or integer > 0xffffffff:
            raise ValueError
        self.integer = integer

    def __int__(self):
        return self.integer

    def __repr__(self):
        return f"<Color({self.integer})>"

    def __str__(self):
        """Return core value in hex form as a string with out any prefix"""
        return hex(self.integer)[2:]

    @classmethod
    def from_name(cls, name: str):
        """
        Return a Color object based on the given color name.

        Only CSS3 extended color keyword names are supported.
        """
        name = name.lower()
        return names.COLORS[name]

    @classmethod
    def from_rgba(cls,
                  red: Union[int, str],
                  green: Union[int, str],
                  blue: Union[int, str],
                  alpha: Union[float, str] = 0):
        """
        Return a Color object from a set of RGBA values
        """
        red_int = cls.to_int(red, skip_types=[float])
        green_int = cls.to_int(green, skip_types=[float])
        blue_int = cls.to_int(blue, skip_types=[float])
        alpha_int = cls.to_int(alpha)
        integer = ((alpha_int << 24) | (red_int << 16)
                   | (green_int << 8) | blue_int)
        return cls(integer)

    @staticmethod
    def to_int(val: Union[float, str],
               skip_types: List[type] = None) -> int:
        """
        Convert float percentage strings, floats in [0,1] or ints in [0,255]
        to equivalent ints.

        Skip conversion if type of val is in skip_types.

        Used to validate color component values.

        Return equivalent int in [0, 255] if conversion is possible.
        Otherwise return None.

        Note: This function is used by the property getter-setters.
        """

        if skip_types is None:
            skip_types = []

        if (type(val) in skip_types) or (type(val) not in [int, float, str]):
            raise ValueError("Invalid value type")

        if isinstance(val, str):
            # For str, the value should be a float percentage
            # with a '%' symbol at the end.
            val = val.strip()
            if val[-1] != "%":
                raise ValueError("String args must be percentages")

            percent_val = float(val[:-1])
            if percent_val < 0 or percent_val > 100:
                raise ValueError("Invalid percentage")
            int_val = round((percent_val/100) * 0xff)

        elif isinstance(val, int):
            # For int, value should be between 0 and 255 (inclusive)
            if val < 0 or val > 255:
                raise ValueError("Invalid value")
            int_val = val

        else:
            # By this point, val must be a float
            # For float, value should be between 0.0 and 1.0 (inclusive)
            if val < 0 or val > 1:
                raise ValueError("Invalid value")
            int_val = round(val * 0xff)

        return int_val

    @property
    def alpha(self) -> float:
        """
        Alpha value ranges from 0 to 1.0
        """
        return (self.integer >> 24) / 0xff

    @alpha.setter
    def alpha(self, val: Union[float, str]):
        val_int = self.to_int(val)
        self.integer = (val_int << 24) | (self.integer & 0x00ffffff)

    @property
    def red(self) -> int:
        """
        Red value ranges from 0 to 255
        """
        return (self.integer >> 16) & 0xff

    @red.setter
    def red(self, val: Union[float, str]):
        val_int = self.to_int(val)
        self.integer = (val_int << 16) | (self.integer & 0xff00ffff)

    @property
    def green(self) -> int:
        """
        Grren value ranges from 0 to 255
        """
        return (self.integer >> 8) & 0xff

    @green.setter
    def green(self, val: Union[float, str]):
        val_int = self.to_int(val)
        self.integer = (val_int << 8) | (self.integer & 0xffff00ff)

    @property
    def blue(self) -> int:
        """
        Blue value ranges from 0 to 255
        """
        return self.integer & 0xff

    @blue.setter
    def blue(self, val: Union[float, str]):
        val_int = self.to_int(val)
        self.integer = val_int | (self.integer & 0xffffff00)
